# -*- coding: utf-8 -*-
#
# Copyright (C) 2021- Soroush Safari <mr.safarii1992@gmail.com>
#
# This file is part of Hyval.
#
# Hyval is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hyval is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with grest.  If not, see <http://www.gnu.org/licenses/>.
#
import base64
from cryptography.fernet import Fernet
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC


def encrypt_keys(setting, value):
    """
            encrypt user credential
    """
    secret_key = PBKDF2HMAC(algorithm=hashes.SHA256(), length=setting['length'], salt=bytes(setting['salt'], 'utf-8'), iterations=setting['iteration'],
                            backend=default_backend()).derive(bytes(setting['key'], 'utf-8'))
    fernet_obj = Fernet(base64.urlsafe_b64encode(secret_key))
    return fernet_obj.encrypt(bytes(value, 'utf-8')).decode("utf-8")


def decrypt_keys(setting, value):
    """
            decrypt user credential
    """

    secret_key = PBKDF2HMAC(algorithm=hashes.SHA256(), length=setting['length'], salt=bytes(setting['salt'], 'utf-8'), iterations=setting['iteration'],
                            backend=default_backend()).derive(bytes(setting['key'], 'utf-8'))
    fernet_obj = Fernet(base64.urlsafe_b64encode(secret_key))
    return fernet_obj.decrypt(bytes(value, 'utf-8')).decode("utf-8")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021- Soroush Safari <mr.safarii1992@gmail.com>
#
# This file is part of Hyval.
#
# Hyval is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hyval is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with grest.  If not, see <http://www.gnu.org/licenses/>.
#

try:
    from django.db import models
    from django.conf import settings
except ImportError:
    raise ImportError(
        "you need to install django or use this in django projects.")

from .utils import encrypt_keys, decrypt_keys
from django.core.exceptions import ValidationError


class HideMyValue(models.CharField):
    description = "A field to save dollars as pennies (int) in db, but act like a float"

    def __init__(self, *args, **kwargs):
        super(HideMyValue, self).__init__(*args, **kwargs)

        self.settting = settings.HIDE_MY_VALUE

    def get_db_prep_value(self, value, *args, **kwargs):
        if value is None:
            return None
        # store encrypted data on database
        return encrypt_keys(setting=self.settting, value=value)

    def to_python(self, value):
        if value is None or isinstance(value, str):
            # retrieve encrypted data from database
            return decrypt_keys(setting=self.settting, value=value)
        try:
            # retrieve encrypted data from database
            return decrypt_keys(setting=self.settting, value=value)
        except (TypeError, ValueError):
            raise ValidationError(
                "This value must be an integer or a string represents an integer.")

    def from_db_value(self, value, expression, connection, context=None):
        return self.to_python(value)

    def formfield(self, **kwargs):
        from django.forms import CharField
        defaults = {'form_class': CharField}
        defaults.update(kwargs)
        return super(HideMyValue, self).formfield(**defaults)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021- Soroush Safari <mr.safarii1992@gmail.com>
#
# This file is part of Hyval.
#
# Hyval is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hyval is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with grest.  If not, see <http://www.gnu.org/licenses/>.
#

import re
from os.path import dirname, join

from setuptools import find_packages, setup

# Extract requirements from requirements.txt
REQUIREMENTS = [
    r.rstrip() for r in open(
        join(
            dirname(__file__),
            'requirements.txt')).readlines()]

print(REQUIREMENTS)
with open(join(dirname(__file__), 'hyval', '__init__.py')) as v_file:
    package_version = re.compile(
        r".*__version__ = '(.*?)'", re.S).match(v_file.read()).group(1)

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="django-hyval",
    version=package_version,
    description="Encryption over django CharField",
    url="https://github.com/coci/django-hyval",
    author="Soroush Safari",
    author_email="soroush.safari1992@gmail.com",
    license="GPLv3",
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=REQUIREMENTS,
    packages=find_packages(),
    classifiers=[
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9"
    ],
    python_requires='>=3.6',
    zip_safe=False
)
